#!/usr/bin/env python3

# This module is part of AsicVerifier and is released under
# the AGPL-3.0-only License: https://opensource.org/license/agpl-v3/

import unittest

from fastapi.testclient import TestClient

from . import (
    URL,
    QUERY_ID,
    X_ROAD_INSTANCE,
    MEMBER_CLASS,
    MEMBER_CODE,
    SUBSYSTEM_CODE,
    ASIC_TYPE,
    ASIC_VERIFIER_RESPONSE
)
from asicverifier.restful_api import RestfulApi


class TestRestfulApi(unittest.TestCase):
    def __init__(self, methodName: str = 'runTest'):
        super().__init__(methodName)
        self.maxDiff = None

    def test_app(self):
        client: TestClient = TestClient(RestfulApi.app())
        self.assertEqual(client.get('/docs').status_code, 200)
        self.assertDictEqual(
            client.get(
                '/',
                params={
                    'security_server_url': URL,
                    'query_id': QUERY_ID,
                    'x_road_instance': X_ROAD_INSTANCE,
                    'member_class': MEMBER_CLASS,
                    'member_code': MEMBER_CODE,
                    'subsystem_code': SUBSYSTEM_CODE,
                    'type': ASIC_TYPE.value
                }
            ).json(),
            ASIC_VERIFIER_RESPONSE
        )



#!/usr/bin/env python3

# This module is part of AsicVerifier and is released under
# the AGPL-3.0-only License: https://opensource.org/license/agpl-v3/

import logging
from os import getenv

from typer import Typer

from . import SUMMARY
from .restful_api import RestfulApi

logging.basicConfig(
    format='%(asctime)s %(levelname)-8s %(message)s',
    level=logging.INFO,
    datefmt='%Y-%m-%d %H:%M:%S'
)
cli: Typer = Typer(
    help=SUMMARY,
    add_completion=False,
    pretty_exceptions_show_locals={
        'true': True, 'false': False
    }.get(getenv('DEV_MODE'), False)
)

for command in [
    RestfulApi.run
]:
    cli.command()(command)

if __name__ == '__main__':
    cli()  # pragma: no cover



#!/usr/bin/env python3

# This module is part of AsicVerifier and is released under
# the AGPL-3.0-only License: https://opensource.org/license/agpl-v3/

from datetime import datetime
from enum import Enum
import re
from urllib.parse import urlencode, urljoin

from dotenv import load_dotenv
from importlib_metadata import PackageMetadata, metadata

load_dotenv()
META_DATA: PackageMetadata = metadata(__name__)
SUMMARY: str = META_DATA['Summary']


def to_datetime(string: str) -> datetime:
    return datetime.strptime(string, r'%a %b %d %H:%M:%S %Z %Y')


def extract_subject_or_issuer(message: str) -> dict:
    return dict([
        element.split('=')
        for element in message.split(', ')
    ])


def extract_asic(message: str) -> dict:
    return {
        'verification': re.search(
            r'Verification (.+)\.', message
        ).group(1),
        **{
            parent: {
                eldest_children: {
                    'subject': subject,
                    'issuer': issuer,
                    'serial_number': serial_number,
                    'valid': {'from': valid_from, 'until': valid_until}
                },
                **youngest_child
            }
            for (
                (parent, eldest_children),
                (subject, issuer, serial_number, valid_from, valid_until),
                youngest_child
            ) in zip(
                (
                    ('signer', 'certificate'),
                    ('ocsp_response', 'signed_by'),
                    ('timestamp', 'signed_by')
                ),
                zip(
                    map(
                        extract_subject_or_issuer,
                        re.findall(
                            r'Subject: (.+)\s+', message
                        )
                    ),
                    map(
                        extract_subject_or_issuer,
                        re.findall(
                            r'Issuer: (.+)\s+', message
                        )
                    ),
                    map(
                        int,
                        re.findall(
                            r'Serial number: (.+)\s+', message
                        )
                    ),
                    map(
                        to_datetime,
                        re.findall(
                            r'Valid from: (.+)\s+', message
                        )
                    ),
                    map(
                        to_datetime,
                        re.findall(
                            r'Valid until: (.+)\s+', message
                        )
                    )
                ),
                (
                    {
                        'id': {
                            key.lower(): value
                            for key, value in [
                                re.search(
                                    r'ID: (.+)\s+', message
                                ).group(1).split(':')
                            ]
                        }
                    },
                    {
                        'produced_at': to_datetime(
                            re.search(
                                r'Produced at: (.+)\s+', message
                            ).group(1)
                        )
                    },
                    {
                        'date': to_datetime(
                            re.search(r'Date: (.+)\s+', message).group(1)
                        )
                    }
                )
            )
        },
        'file': [
            {'path': path, 'digist': digist, 'status': status}
            for path, digist, status in re.findall(
                r'digest for \"(.+)\" is: (.+) \((.+)\)', message
            )
        ]
    }


class AsicType(str, Enum):
    REQUEST: str = 'request'
    RESPONSE: str = 'response'


def asicverifier(
    security_server_url: str,
    query_id: str,
    x_road_instance: str,
    member_class: str,
    member_code: str,
    subsystem_code: str,
    type: AsicType = AsicType.REQUEST
) -> dict:
    verificationconf_url: str = urljoin(
        security_server_url, 'verificationconf'
    )
    asic_url: str = '{url}?unique&{type}&{params}'.format(
        url=urljoin(security_server_url, 'asic'),
        type={
            enum.value: f'{enum.value}Only'
            for enum in AsicType
        }[type.value],
        params=urlencode({
            'queryId': query_id,
            'xRoadInstance': x_road_instance,
            'memberClass': member_class,
            'memberCode': member_code,
            'subsystemCode': subsystem_code
        })
    )
    return {'verificationconf_url': verificationconf_url, 'asic_url': asic_url}



#!/usr/bin/env python3

# This module is part of AsicVerifier and is released under
# the AGPL-3.0-only License: https://opensource.org/license/agpl-v3/

from datetime import datetime
from os import getenv
from typing import List, Union

from fastapi import APIRouter, FastAPI
from fastapi.middleware.cors import CORSMiddleware
from pydantic import (
    BaseModel,
    DirectoryPath,
    EmailStr,
    Field,
    FilePath,
    NonNegativeInt
)
import uvicorn

from . import asicverifier, META_DATA, SUMMARY


class AsicSignerId(BaseModel):
    subsystem: DirectoryPath


class AsicSignValid(BaseModel):
    from_: datetime = Field(alias='from')
    until: datetime


class AsicOcsp(BaseModel):
    CN: str
    O: str


class AsicSign(BaseModel):
    subject: AsicOcsp
    issuer: AsicOcsp
    serial_number: NonNegativeInt
    valid: AsicSignValid


class AsicSignerCertificateSubject(AsicOcsp):
    C: str


class AsicSignerCertificate(AsicSign):
    subject: AsicSignerCertificateSubject


class AsicSigner(BaseModel):
    certificate: AsicSignerCertificate
    id: AsicSignerId


class AsicOcspResponse(BaseModel):
    signed_by: AsicSign
    produced_at: datetime


class AsicTimeStampSignByIssuer(AsicSignerCertificateSubject, BaseModel):
    ST: str
    L: str
    EMAILADDRESS: EmailStr
    OU: str


class AsicTimeStampSignBySubject(AsicTimeStampSignByIssuer):
    oid_2_5_4_13: str = Field(alias='OID.2.5.4.13')


class AsicTimeStampSignBy(AsicSign):
    subject: AsicTimeStampSignBySubject
    issuer: AsicTimeStampSignByIssuer


class AsicTimeStamp(BaseModel):
    signed_by: AsicTimeStampSignBy
    date: datetime


class AsicFile(BaseModel):
    path: FilePath
    digist: str
    status: str


class Asic(BaseModel):
    verification: str
    signer: AsicSigner
    ocsp_response: AsicOcspResponse
    timestamp: AsicTimeStamp
    file: List[AsicFile]


class RestfulApi:
    @staticmethod
    def app() -> FastAPI:
        RESTFUL_API_PATH: str = getenv('RESTFUL_API_PATH', '/')

        if RESTFUL_API_PATH.endswith('/'):
            RESTFUL_API_PATH = RESTFUL_API_PATH[:-1]

        api: FastAPI = FastAPI(
            title=SUMMARY,
            version=META_DATA['Version'],
            docs_url=f'{RESTFUL_API_PATH}/docs',
            redoc_url=f'{RESTFUL_API_PATH}/redoc',
            openapi_url=f'{RESTFUL_API_PATH}/openapi.json'
        )
        api.add_middleware(
            CORSMiddleware,
            allow_origins=[
                'http://0.0.0.0',
                'http://localhost',
                'http://localhost:8080'
            ],
            allow_credentials=True,
            allow_methods=['*'],
            allow_headers=['*']
        )
        router = APIRouter()
        router.get(
            '/', name='verifier', response_model=Union[Asic, dict]
        )(asicverifier)

        api.include_router(router, prefix=RESTFUL_API_PATH)
        return api

    @staticmethod
    def run(
        host: str = '0.0.0.0', port: int = 80, reload: bool = False
    ):
        'RESTful API'

        uvicorn.run(
            f'{__name__}:RestfulApi.app',
            host=host,
            port=port,
            reload=reload,
            factory=True
        )  # pragma: no cover
